﻿using System.Linq;
using System.Collections.Generic;

using Microsoft.Msagl.Core.Geometry;
using Microsoft.Msagl.Core.Geometry.Curves;
using Microsoft.Msagl.Core.Layout;
using Microsoft.Msagl.Prototype.Phylo;

using DevExpress.Diagram.Core;
using DevExpress.Diagram.Core.Layout;

namespace TestLibrary
{
    /// <summary>
    /// MSAGL 기하 그래프 헬퍼
    /// </summary>
    public static class MSAGLGeometryGraphHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 노드 위치 정보 구하기 - GetNodePositionInfo(geometryGraph)

        /// <summary>
        /// 노드 위치 정보 구하기
        /// </summary>
        /// <param name="geometryGraph">기하 그래프</param>
        /// <returns>노드 위치 정보</returns>
        public static IEnumerable<PositionInfo<IDiagramItem>> GetNodePositionInfo(GeometryGraph geometryGraph)
        {
            return geometryGraph.Nodes.Select
            (
                node => new PositionInfo<IDiagramItem>
                (
                    (IDiagramItem)node.UserData,
                    Converter.GetPoint(node.BoundingBox.LeftTop)
                )
            );
        }

        #endregion
        #region PHYLO 트리 구하기 - GetPhyloTree(graph)

        /// <summary>
        /// PHYLO 트리 구하기
        /// </summary>
        /// <param name="graph">그래프</param>
        /// <returns>PHYLO 트리</returns>
        public static PhyloTree GetPhyloTree(Graph<IDiagramItem> graph)
        {
            PhyloTree phyloTree = new PhyloTree();

            foreach(IDiagramItem node in graph.Nodes)
            {
                AddNode(phyloTree, node);
            }

            foreach(Edge<IDiagramItem> edge in graph.Edges)
            {
                AddPhyloTreeEdge(phyloTree, edge.From, edge.To, edge.Weight);
            }

            return phyloTree;
        }

        #endregion
        #region 기하 그래프 구하기 - GetGeometryGraph(graph)

        /// <summary>
        /// 기하 그래프 구하기
        /// </summary>
        /// <param name="graph">그래프</param>
        /// <returns>기하 그래프</returns>
        public static GeometryGraph GetGeometryGraph(Graph<IDiagramItem> graph)
        {
            GeometryGraph geometryGraph = new GeometryGraph();

            foreach(IDiagramItem node in graph.Nodes)
            {
                AddNode(geometryGraph, node);
            }

            foreach(Edge<IDiagramItem> edge in graph.Edges)
            {
                AddEdge(geometryGraph, edge.From, edge.To, edge.Weight);
            }

            return geometryGraph;
        }

        #endregion
        #region 노드 추가하기 - AddNode(geometryGraph, item)

        /// <summary>
        /// 노드 추가하기
        /// </summary>
        /// <param name="geometryGraph">기하 그래프</param>
        /// <param name="item">항목</param>
        /// <returns>노드</returns>
        public static Node AddNode(GeometryGraph geometryGraph, IDiagramItem item)
        {
            Node node = geometryGraph.FindNodeByUserData(item);

            if(node == null)
            {
                node = new Node(GetCurve(item), item);

                geometryGraph.Nodes.Add(node);
            }

            return node;
        }

        #endregion
        #region 에지 추가하기 - AddEdge(geometryGraph, parentNode, childNode, weight)

        /// <summary>
        /// 에지 추가하기
        /// </summary>
        /// <param name="geometryGraph">기하 그래프</param>
        /// <param name="parentNode">부모 노드</param>
        /// <param name="childNode">자식 노드</param>
        /// <param name="weight">가중치</param>
        public static void AddEdge(GeometryGraph geometryGraph, IDiagramItem parentNode, IDiagramItem childNode, double weight)
        {
            Edge edge = new Edge
            (
                AddNode(geometryGraph, parentNode),
                AddNode(geometryGraph, childNode )
            );

            edge.Weight = (int)weight;

            geometryGraph.Edges.Add(edge);
        }

        #endregion
        #region PHYLO 트리 에지 추가하기 - AddPhyloTreeEdge(phyloTree, parentNode, childNode, weight)

        /// <summary>
        /// PHYLO 트리 에지 추가하기
        /// </summary>
        /// <param name="phyloTree">PHYLO 트리</param>
        /// <param name="parentNode">부모 노드</param>
        /// <param name="childNode">자식 노드</param>
        /// <param name="weight">가중치</param>
        public static void AddPhyloTreeEdge(PhyloTree phyloTree, IDiagramItem parentNode, IDiagramItem childNode, double weight = 1)
        {
            PhyloEdge edge = new PhyloEdge
            (
                AddNode(phyloTree, parentNode),
                AddNode(phyloTree, childNode )
            );
            
            edge.Weight = (int)weight;

            phyloTree.Edges.Add(edge);
        }

        #endregion
        #region 커브 구하기 - GetCurve(item)

        /// <summary>
        /// 커브 구하기
        /// </summary>
        /// <param name="item">항목</param>
        /// <returns>커브</returns>
        public static ICurve GetCurve(IDiagramItem item)
        {
            return CurveFactory.CreateRectangle(item.ActualSize.Width, item.ActualSize.Height, new Point());
        }

        #endregion
    }
}